@@ -1,8 +1,11 @@
 import pandas as pd
 import numpy as np
-import warnings
+import seaborn as sns
+import matplotlib.pyplot as plt
 from tqdm import tqdm
-warnings.filterwarnings('ignore')
+from sklearn.metrics import roc_curve, roc_auc_score
+from matplotlib.ticker import MultipleLocator
+
 
 def bin_badrate(df, col_name, target):
     """
@@ -741,9 +744,6 @@ def chi2_cutting_discrete(df_data, feat_list, target,
         raise Exception('Input is not a dataframe.')
     elif df_data.shape[0] == 0:
         raise Exception('The dataframe has a row num of 0.')
-    # 判断分箱特征列表是否为空
-    elif (len(feat_list) == 0) or (feat_list is None):
-        raise Exception('Empty list is inputed.')
     # 判断最大分箱数max_intervals是否为正整数
     elif (max_intervals < 0) or not isinstance(max_intervals, int):
         raise Exception('Max_intervals is incorrect.')
@@ -751,7 +751,7 @@ def chi2_cutting_discrete(df_data, feat_list, target,
     elif (min_pnt <= 0) or (min_pnt >= 1):
         raise Exception('Min_pnt is incorrect.')
 
-    # 该函数不允许有序性存在特殊值
+    # 该函数不允许有序型存在特殊值
     for key in discrete_order.keys():
         if key in special_feat_val.keys():
             str_exception = "'%s' is orderly feature and exists in dictionary of speacial values." % key
@@ -1003,9 +1003,6 @@ def chi2_cutting_continuous(df_data, feat_list, target,
         raise Exception('Input is not a dataframe.')
     elif df_data.shape[0] == 0:
         raise Exception('The dataframe has a row num of 0.')
-    # 判断分箱特征列表是否为空
-    elif (len(feat_list) == 0) or (feat_list is None):
-        raise Exception('Empty list is inputed.')
     # 判断最大分箱数max_intervals是否为正整数
     elif (max_intervals < 0) or not isinstance(max_intervals, int):
         raise Exception('Max_intervals is incorrect.')
@@ -1203,3 +1200,657 @@ def chi2_cutting_continuous(df_data, feat_list, target,
 
     return dict_contin_feat_to_bins, dict_contin_iv, dict_contin_woe
 
+def model_roc_auc(y_true, y_proba, is_plot=False, dict_plot_params=None):
+    """
+    绘制roc曲线
+    输入X_train_woe, 含预测结果和预测概率的训练集
+    输入is_plot, 是否绘图
+
+    Parameters:
+    ----------
+    y_true: list, 实际的y(默认1为正例)
+
+    y_proba: list, 预测的y的probability(概率)
+
+    is_plot: boolean, 是否要绘图
+
+    dict_plot_params: dict, 对plot的图像的参数
+
+    Returns:
+    -------
+    auc: float, auc值
+    """
+    # 转换类型
+    y_true, y_proba = list(y_true), list(y_proba)
+
+    # 处理参数
+    if dict_plot_params is None:
+        dict_plot_params = {'fontsize': 15,
+                            'figsize': (15, 8),
+                            'linewidth': 5}
+
+    if 'fontsize' in dict_plot_params.keys():
+        fontsize = dict_plot_params['fontsize']
+    else:
+        fontsize = 15
+
+    if 'figsize' in dict_plot_params.keys():
+        figsize = dict_plot_params['figsize']
+    else:
+        figsize = (15, 8)
+
+    if 'linewidth' in dict_plot_params.keys():
+        linewidth = dict_plot_params['linewidth']
+    else:
+        linewidth = 5
+
+    # 计算auc
+    auc = roc_auc_score(y_true=y_true, y_score=y_proba)
+
+    # 是否绘图
+    if is_plot:
+        fontdict = {'fontsize': fontsize}
+
+        # 绘制roc曲线
+        fpr, tpr, thresholds = roc_curve(y_true=y_true,
+                                         y_score=y_proba,
+                                         pos_label=1)
+        fig, ax = plt.subplots(figsize=figsize)
+
+        # 绘制roc
+        ax.plot(fpr, tpr, label='ROC Curve(AUC=%.2F)' % auc, linewidth=linewidth)
+
+        # 绘制(0,0) (1,1)直线
+        ax.plot([0, 1], [0, 1], linestyle='--', c='r', linewidth=linewidth)
+
+        ax.set_title('Receiver Operating Characteristic', fontdict=fontdict)
+        ax.set_xlim([0, 1])
+        ax.set_ylim([0, 1.05])
+        ax.set_xlabel('False Positive Rate', fontdict=fontdict)
+        ax.set_ylabel('True Positive Rate', fontdict=fontdict)
+        ax.legend(loc='best', fontsize='large')
+    # 返回结果
+    return auc
+
+
+def model_ks(y_true, y_pred, y_proba, is_plot=True, dict_plot_params=None):
+    """
+    计算模型的ks(Kolmogorov-Smirnov)
+
+    ks: <0 模型错误, 0~0.2 模型较差, 0.2~0.3 模型可用, >0.3 模型预测性较好
+
+    ks计算公式为 ks = max(cum(Bi)/ Bad(total) - cum(Gi) / Good(total))
+
+    Parameters:
+    ----------
+    y_true: list, 真实的y
+
+    y_pred: list,预测的y
+
+    y_proba: list, 预测的y的概率
+
+    dict_plot_params: dict, 绘图的参数
+
+    Returns:
+    -------
+    ks: 模型的ks值
+
+
+    """
+    # 转换类型
+    y_true, y_pred, y_proba = list(y_true), list(y_pred), list(y_proba)
+
+    # 处理参数
+    if dict_plot_params is None:
+        dict_plot_params = {'fontsize': 15,
+                            'figsize': (15, 8),
+                            'linewidth': 2}
+
+    if 'fontsize' in dict_plot_params.keys():
+        fontsize = dict_plot_params['fontsize']
+    else:
+        fontsize = 15
+
+    if 'figsize' in dict_plot_params.keys():
+        figsize = dict_plot_params['figsize']
+    else:
+        figsize = (15, 8)
+
+    if 'linewidth' in dict_plot_params.keys():
+        linewidth = dict_plot_params['linewidth']
+    else:
+        linewidth = 2
+
+    # 创建df, 计算总样本数、bad样本数、good样本数
+    df = pd.DataFrame({'y_true': y_true, 'y_pred': y_pred, 'y_proba': y_proba})
+    cnt_total = df['y_true'].shape[0]
+    cnt_bad = df.loc[df['y_true'] == 1].shape[0]
+    cnt_good = cnt_total - cnt_bad
+
+    # 对df基于proba进行降序排序
+    df = df.sort_values(by='y_proba', ascending=False)
+
+    # 计算总样本(排序后的)下的样本数占比
+    df['cum_pnt_total'] = [i / cnt_total for i in list(range(1, cnt_total + 1))]
+
+    # 计算好样本(排序后的)下的样本数占比
+    df['cum_pnt_good'] = (df['y_true'] == 0).cumsum() / cnt_good
+
+    # 计算坏样本(排序后的)下的样本数占比
+    df['cum_pnt_bad'] = (df['y_true'] == 1).cumsum() / cnt_bad
+
+    # 计算差值的绝对值
+    df['cum_diff_value'] = abs(df['cum_pnt_bad'] - df['cum_pnt_good'])
+
+    # 计算ks值
+    ks = df['cum_diff_value'].max()
+
+    # 绘制ks曲线
+    if is_plot:
+        ## 取得ks下的index, 计算在该idx下的cum_pnt_total
+        idxmax = df['cum_diff_value'].idxmax()
+        x_ks = df.loc[idxmax, 'cum_pnt_total']
+
+        ## 绘图
+        fig, ax = plt.subplots(figsize=figsize)
+
+        ax.plot(df['cum_pnt_total'], df['cum_pnt_bad'], linewidth=linewidth, label='Bad')
+        ax.plot(df['cum_pnt_total'], df['cum_pnt_good'], linewidth=linewidth, label='Good')
+        ax.plot(df['cum_pnt_total'], df['cum_diff_value'], linewidth=linewidth, label='K-S Curve')
+        ax.plot([x_ks], [ks], 'o')
+        ax.annotate('K-S Statistic(%.2f)' % ks, xy=(x_ks, ks),
+                    xytext=(x_ks, ks / 1.5),
+                    fontsize=fontsize,
+                    arrowprops=dict(arrowstyle='fancy', facecolor='red'))
+
+        ax.grid(linewidth=0.4, linestyle='--')
+        ax.legend(loc='best', fontsize='large')
+        ax.set_title('K-S Curve', fontsize=fontsize)
+
+        ax.set_xlim((0, 1))
+        ax.set_ylim((0, 1))
+
+    return ks
+
+
+def model_gini(y_true, y_proba, is_plot=False, dict_plot_params=None):
+    """
+    计算Gini系数
+
+    先基于score进行升序排序
+    Gini系数图中, 横坐标是累计的总数或者总数占比
+    纵坐标是累计的正例数或者正例数占比(洛伦兹曲线)
+    45%直线代表无差异, 洛伦兹曲线越靠近该直线, 表示区分度越低
+
+    gini不纯度，越小越纯，区分度越好
+
+    好坏样本占比差异较大时, 坏样本很少, 此时洛伦兹曲线和ROC曲线横纵轴取值基本一致, 曲线基本一致,
+    此时可以使用近似计算gini=2*AUC - 1
+
+    Parameters:
+    ----------
+    y_true: list, 真实的y
+
+    y_proba: list, y的预测概率
+
+    is_plot: boolean, 是否绘制Gini曲线
+
+    dict_plot_params: dict, 绘图的参数
+
+    Returns:
+    -------
+    gini: 最终的gini系数
+
+    """
+    # 转换类型
+    y_true, y_proba = list(y_true), list(y_proba)
+
+    # 处理参数
+    if dict_plot_params is None:
+        dict_plot_params = {'fontsize': 15,
+                            'figsize': (12, 12),
+                            'linewidth': 2}
+
+    if 'fontsize' in dict_plot_params.keys():
+        fontsize = dict_plot_params['fontsize']
+    else:
+        fontsize = 15
+
+    if 'figsize' in dict_plot_params.keys():
+        figsize = dict_plot_params['figsize']
+    else:
+        figsize = (12, 12)
+
+    if 'linewidth' in dict_plot_params.keys():
+        linewidth = dict_plot_params['linewidth']
+    else:
+        linewidth = 2
+
+    # 构建df
+    df = pd.DataFrame({'y_true': y_true, 'y_proba': y_proba})
+
+    # 样本个数, 坏样本个数
+    cnt_total = df.shape[0]
+    cnt_bad = df[df['y_true'] == 1].shape[0]
+
+    # 基于score进行升序
+    df = df.sort_values(by='y_proba', ascending=True)
+
+    # 计算总样本(排序后的)下的样本数占比
+    df['cum_pnt_total'] = [i / cnt_total for i in list(range(1, cnt_total + 1))]
+
+    # 计算坏样本(排序后的)下的样本数占比
+    df['cum_pnt_bad'] = (df['y_true'] == 1).cumsum() / cnt_bad
+
+    # 计算45°线之间的差值
+    df['cum_pnt_diff'] = df['cum_pnt_total'] - df['cum_pnt_bad']
+
+    # 计算A, B, G, 用梯形拟合计算B的面积
+    area_b = np.trapz(df['cum_pnt_bad'], df['cum_pnt_total'])
+
+    # 计算面积A
+    area_a = 0.5 - area_b
+
+    # 计算GINI
+    gini = area_a / (area_a + area_b)
+
+    if is_plot:
+        fontdict = {'fontsize': fontsize}
+        fig, ax = plt.subplots(figsize=figsize)
+        ax.stackplot(df['cum_pnt_total'],
+                     df['cum_pnt_bad'],
+                     df['cum_pnt_diff'],
+                     colors=['#7DB2FA', '#AFC4D6', 'black'])
+        ax.plot(df['cum_pnt_total'],
+                df['cum_pnt_bad'],
+                color='black',
+                linewidth=linewidth,
+                label='Lorenz Curve(Gini=%.2f)' % gini)
+        ax.plot(df['cum_pnt_total'],
+                df['cum_pnt_total'],
+                color='black',
+                linewidth=linewidth)
+
+        ax.set_xlim((0, 1))
+        ax.set_ylim((0, 1))
+        ax.set_title('Lorenz Curve and Gini coefficient', fontsize=fontsize)
+        ax.set_xlabel('Cumulative Pct. of Total', fontdict=fontdict)
+        ax.set_ylabel('Cumulative Pct. of Bad', fontdict=fontdict)
+        ax.text(0.4, 0.45, 'Line of Equality (45 Degree)', rotation=45, fontsize=fontsize, fontweight='bold')
+        ax.text(0.4, 0.2, 'Lorenz Curve', rotation=45, fontsize=fontsize, fontweight='bold')
+        ax.text(0.8, 0.7, 'A', fontsize=fontsize + 10, fontweight='bold')
+        ax.text(0.9, 0.1, 'B', fontsize=fontsize + 10, fontweight='bold')
+        ax.legend(fontsize='xx-large', loc='best')
+
+    return gini
+
+
+def model_psi(score_train, score_test, n_split=10):
+    """
+
+    计算模型稳定性PSI
+    基于分数进行n_split等分, 再计算PSI
+    PSI的公式为PSI=sum((actual_pnt - expected_pnt) *  ln(actual_pnt / expected_pnt))
+    psi的经验值,psi < 0.1:稳定性高，无需更新模型;
+    # 0.1 <= psi <0.25:稳定性一般,需要进一步研究;psi >= 0.25:稳定性差,需要更新模型
+
+    Parameters:
+    ----------
+    score_train: list, 训练集的分数分布(训练集的样本数和测试集是不一定相等的！！！)
+
+    score_test: list, 测试集的分数分布(训练集的样本数和测试集是不一定相等的！！！)
+
+    Returns:
+    -------
+    psi: float, psi index
+
+    """
+    # score分组dict
+    dict_score_interval = {}
+
+    # 构建df
+    df_train = pd.DataFrame({'score': score_train})
+    df_test = pd.DataFrame({'score': score_test})
+
+    # 计算各数据集的样本数
+    cnt_train = df_train.shape[0]
+    cnt_test = df_test.shape[0]
+
+    # 从验证集开始采集分数段
+    list_scores = list(set(df_train['score']))
+
+    # 分数最大值、最小值
+    max_score = max(list_scores)
+    min_score = min(list_scores)
+
+    # 采集的分数分段
+    interval_scores = (max_score - min_score) / n_split
+
+    # 采集刻度值
+    list_score_split = sorted([int(min_score + i * interval_scores) for i in range(1, n_split)])
+
+    # 形成分组dict, 注意需要手工添加最后一组
+    for i in range(len(list_score_split)):
+        if i == 0:
+            label_l = '[0,'
+            label_r = str(list_score_split[0]) + ')'
+            label = label_l + label_r
+            dict_score_interval[label] = 0
+        else:
+            label_l = '[' + str(list_score_split[i - 1]) + ','
+            label_r = str(list_score_split[i]) + ')'
+            label = label_l + label_r
+            dict_score_interval[label] = i
+
+    dict_score_interval['[%s, +Inf)' % max(list_score_split)] = n_split - 1
+    # 做个升序sorted
+    dict_score_interval = {k: v for k, v in sorted(dict_score_interval.items(), key=lambda x: x[1])}
+
+    # 映射训练集和测试机的分数区间
+    df_train['score_interval'] = df_train['score'].apply(lambda x: value_to_intervals(x, dict_score_interval))
+    df_test['score_interval'] = df_test['score'].apply(lambda x: value_to_intervals(x, dict_score_interval))
+
+    # 基于分数区间映射groupno
+    df_train['score_groupno'] = df_train['score_interval'].map(dict_score_interval)
+    df_test['score_groupno'] = df_test['score_interval'].map(dict_score_interval)
+
+    # 组成训练集psi统计用的dataframe
+    df_psi_train = df_train.groupby(['score_groupno', 'score_interval'], as_index=False)['score'].count()
+    df_psi_train['actual_pnt'] = df_psi_train['score'] / cnt_train
+    df_psi_train.drop('score', axis=1, inplace=True)
+
+    # 组成测试集psi统计用的dataframe
+    df_psi_test = df_test.groupby(['score_groupno', 'score_interval'], as_index=False)['score'].count()
+    df_psi_test['expected_pnt'] = df_psi_test['score'] / cnt_test
+    df_psi_test.drop('score', axis=1, inplace=True)
+
+    # 组成df_psi
+    df_psi = pd.merge(left=df_psi_train, right=df_psi_test,
+                      how='left',
+                      on=['score_groupno', 'score_interval'])
+
+    # 可能存在测试集不存在分组的情况, 需要填充, 注意该值在后续还需要参与分母计算, 这里填充个接近0的量
+    df_psi['expected_pnt'] = df_psi['expected_pnt'].fillna(1e-9)
+
+    # 计算其他指标
+    df_psi['diff_ac_exp_pnt'] = df_psi['actual_pnt'] - df_psi['expected_pnt']
+    df_psi['div_ac_exp_pnt'] = df_psi['actual_pnt'] / df_psi['expected_pnt']
+    df_psi['ln_ac_exp_pnt'] = np.log(df_psi['div_ac_exp_pnt'])
+    df_psi['index'] = df_psi['diff_ac_exp_pnt'] * df_psi['ln_ac_exp_pnt']
+
+    # 计算psi
+    psi = df_psi['index'].sum()
+
+    return psi, df_psi
+
+
+def model_lift(y_true, y_score, n_split=10, is_plot=False, dict_plot_params=None):
+    """
+    计算LIFT提升度, 比较用模型与不用模型的提升
+    计算模型在该P阈值下的精度Precision = TP/(TP+FP)
+    由模型定义可知, 大于p值的会被分类正类(1),小于的则会成为负类(0)
+    所以经过proba排序后, 逻辑类似TPR,FPR
+    此时模型预测的坏样本数其实就是样本数
+
+    解读lift:
+    在样本内,样本数为M,含有N个正类需要抓取, 此时抓取概率为N/M,随机抓取
+    若以p_thres作为阈值,在大于阈值的样本(sub_sample)同样是抓取N个正类,此时由于范围小了,自然抓取概率变大,变大为N/(FP+TP)
+    则lift为此时精度与实际比率的提升率
+    从公式来看,可以理解为在M中抓,和在(FP+TP)内抓取N个正类, LIFT其实也就是M/(FP+TP), 即抓取面积的比率
+    depth可以理解为样本的百分比或者某个分数、某个概率阈值
+    如果阈值设的很大,即depth很小,这个时候lift很大, 为了展现, 一般以10% * N 作为阈值点
+
+    输入X 输入的含y, y_pred, y_proba的集合
+    输入targetname, 目标变量名
+    输入n_split, 分组数
+
+    Parameters:
+    ----------
+
+    Returns:
+    -------
+    Just plot.
+
+    """
+    # 转换类型
+    y_true, y_score = list(y_true), list(y_score)
+
+    # 处理参数
+    if dict_plot_params is None:
+        dict_plot_params = {'fontsize': 15,
+                            'figsize': (15, 12),
+                            'linewidth': 3,
+                            'width': 0.35}
+
+    if 'fontsize' in dict_plot_params.keys():
+        fontsize = dict_plot_params['fontsize']
+    else:
+        fontsize = 15
+
+    if 'figsize' in dict_plot_params.keys():
+        figsize = dict_plot_params['figsize']
+    else:
+        figsize = (15, 12)
+
+    if 'linewidth' in dict_plot_params.keys():
+        linewidth = dict_plot_params['linewidth']
+    else:
+        linewidth = 3
+
+    if 'width' in dict_plot_params.keys():
+        width = dict_plot_params['width']
+    else:
+        width = 0.35
+
+    # 构建df
+    df = pd.DataFrame({'y_true': y_true, 'score': y_score})
+
+    # 基于score进行降序排序
+    df = df.sort_values(by='score', ascending=True).reset_index(drop=True)
+
+    # 总样本数
+    cnt_total = df.shape[0]
+    # 每个分组样本数
+    cnt_perbin = np.ceil(cnt_total / n_split)
+    # 先计算得到阈值的index
+    list_index = []
+    for i in range(1, n_split + 1):
+        if i == n_split:
+            list_index.append(cnt_total - 1)
+        else:
+            list_index.append(int(i * cnt_perbin))
+
+    list_index = sorted(list_index)
+
+    # 计算TP,FP,DEPTH,FPR, BADRATE_ACTUAL,LIFT
+    df['true_positive'] = (df['y_true'] == 1).cumsum()
+    df['false_positive'] = (df['y_true'] == 0).cumsum()
+    df['fpr'] = df['true_positive'] / (df['true_positive'] + df['false_positive'])
+    df['depth'] = [i / cnt_total for i in range(1, cnt_total + 1)]
+    df['badrate_actual'] = (df['y_true'] == 1).cumsum() / cnt_total
+    df['lift_cum'] = df['fpr'] / df['badrate_actual']
+
+    # 产生score分组区间
+    list_scores = df.loc[list_index, 'score'].to_list()
+    dict_score_interv = {}
+    for i in range(len(list_index)):
+        if i == 0:
+            label = '[0,%s]' % str(list_scores[i])
+            dict_score_interv[label] = i
+        else:
+            label = '(%s,%s]' % (str(list_scores[i - 1]), str(list_scores[i]))
+            dict_score_interv[label] = i
+
+    df['score_bin'] = df['score'].apply(lambda x: value_to_intervals(x, dict_score_interv))
+    df['score_bin_no'] = df['score_bin'].map(dict_score_interv)
+
+    # 汇总 & 计算模型累积值
+    lift_regroup = df.groupby(['score_bin_no', 'score_bin'])['y_true'].agg(['count', 'sum'])
+    lift_regroup = lift_regroup.rename(columns={'count': 'total_cnt', 'sum': 'bad_cnt'}).reset_index()
+
+    lift_regroup['bad_pnt_int'] = lift_regroup['bad_cnt'] / lift_regroup['bad_cnt'].sum()
+    lift_regroup['total_pnt'] = lift_regroup['total_cnt'] / cnt_total
+    lift_regroup['bad_pnt_int_cum'] = lift_regroup['bad_pnt_int'].cumsum()
+    lift_regroup['total_pnt_cum'] = lift_regroup['total_pnt'].cumsum()
+    lift_regroup['lift'] = lift_regroup['bad_pnt_int_cum'] / lift_regroup['total_pnt_cum']
+
+    if is_plot:
+        fig, ax = plt.subplots(2, 1, figsize=figsize)
+
+        # 绘制累积LIFT图
+        ax[0].plot(df.loc[list_index, 'depth'], df.loc[list_index, 'lift_cum'],
+                   label='Lift(Model)',
+                   linewidth=linewidth,
+                   marker='o')
+
+        ax[0].plot(df.loc[list_index, 'depth'], [1] * n_split,
+                   label='Lift(Random)',
+                   linewidth=linewidth,
+                   marker='o',
+                   color='r')
+
+        ax[0].set_title('Cumulative Lift Chart', fontsize=fontsize)
+        ax[0].set_xlabel('Depth', fontsize=fontsize)
+        ax[0].set_ylabel('Lift', fontsize=fontsize)
+        ax[0].legend(loc='best', fontsize='x-large')
+        ax[0].grid()
+        ax[0].xaxis.set_major_locator(MultipleLocator(0.1))
+        ax[0].yaxis.set_major_locator(MultipleLocator(1))
+
+        # 绘制分数分组LIFT图
+        ax[1].bar(lift_regroup['score_bin_no'] - width / 2, lift_regroup['bad_pnt_int'],
+                  width=width,
+                  color='#7DB2FA',
+                  label='Bad_pct(Model)')
+        ax[1].bar(lift_regroup['score_bin_no'] + width / 2, lift_regroup['total_pnt'],
+                  width=width,
+                  color='#AFC4D6',
+                  label='Bad_pct(Random)')
+        ax_twinx_1 = ax[1].twinx()
+        ax_twinx_1.plot(lift_regroup['score_bin_no'], lift_regroup['lift'], label='Lift', marker='o', color='r')
+        ax[1].set_title('Lift Chart', fontsize=fontsize)
+        ax[1].set_xlabel('Score Group', fontsize=fontsize)
+        ax[1].set_ylabel('Pct. of Bad', fontsize=fontsize)
+        ax[1].set_xticks(range(n_split))
+        ax[1].set_xticklabels(list(lift_regroup['score_bin']), fontsize=fontsize - 5)
+        ax_twinx_1.set_ylabel('Lift', fontsize=fontsize)
+        ax[1].grid()
+        ax[1].legend(loc='upper right', fontsize='medium')
+
+        plt.legend(['Lift'], loc='upper left', fontsize='large')
+
+    return lift_regroup
+
+
+def plot_score_distribution(y, score, dict_plot_params=None):
+    """
+    绘制score的频数分布, score的01频数分布, badrate与score的曲线
+
+    Parameters:
+    ----------
+    y: list, 数据集的目标变量(Y标), 可以是任意集合的
+
+    scores: list, 数据集的分数
+
+
+    Returns:
+    -------
+    Just plot.
+
+    """
+    # 转换类型
+    y, score = list(y), list(score)
+
+    # 处理参数
+    if dict_plot_params is None:
+        dict_plot_params = {'fontsize': 15,
+                            'figsize': (15, 20),
+                            'linewidth': 3,
+                            'markersize': 12}
+
+    if 'fontsize' in dict_plot_params.keys():
+        fontsize = dict_plot_params['fontsize']
+    else:
+        fontsize = 15
+
+    if 'figsize' in dict_plot_params.keys():
+        figsize = dict_plot_params['figsize']
+    else:
+        figsize = (15, 20)
+
+    if 'linewidth' in dict_plot_params.keys():
+        linewidth = dict_plot_params['linewidth']
+    else:
+        linewidth = 3
+
+    if 'markersize' in dict_plot_params.keys():
+        markersize = dict_plot_params['markersize']
+    else:
+        markersize = 12
+
+    # 构造df
+    df = pd.DataFrame({'y': y, 'score': score})
+
+    # 绘制一个3X1的画布
+    fig, ax = plt.subplots(3, figsize=figsize)
+
+    fontdict = {'fontsize': fontsize}
+
+    # 第一个绘制评分频数分布
+    sns.kdeplot(df['score'], ax=ax[0], linewidth=linewidth)
+    ax[0].grid(linestyle='--', alpha=0.5)
+    ax[0].set_title('Distribution of scores', fontsize=fontsize)
+    ax[0].set_xlabel('Score', fontdict=fontdict)
+    ax[0].set_ylabel('Density', fontdict=fontdict)
+
+    # 第二个绘制0,1两类的频数分布
+    sns.kdeplot(df.loc[df['y'] == 0, 'score'], ax=ax[1], label='y=0', linewidth=linewidth)
+
+    sns.kdeplot(df.loc[df['y'] == 1, 'score'], ax=ax[1], label='y=1', linewidth=linewidth)
+
+    ax[1].set_title('Distibution of scores(0,1)', fontsize=fontsize)
+    ax[1].grid(linestyle='--', alpha=0.5)
+    ax[1].set_xlabel('Score', fontdict=fontdict)
+    ax[1].set_ylabel('Density', fontdict=fontdict)
+    ax[1].legend(fontsize='large')
+
+    # 第三个绘制badrate的频数分布
+    score_min, score_max = df['score'].min() // 100 * 100, df['score'].max() // 100 * 100
+
+    x_linsrange = np.linspace(df['score'].min(), df['score'].max(), 20)
+
+    df['score_bins'] = pd.cut(df['score'], bins=x_linsrange)
+
+    regroup_badrate = bin_badrate(df, col_name='score_bins', target='y')
+    ## 可能存在有空箱吗？ 以防万一兜底
+    regroup_badrate['bad_rate'].fillna(0, inplace=True)
+
+    sns.histplot(df['score'],
+                 bins=x_linsrange,
+                 label='Count',
+                 kde=False,
+                 ax=ax[2],
+                 color='#A5C8E1',
+                 linewidth=0.5)
+
+    ax[2].set_title('Distibution of score groups & Badrate', fontsize=fontsize)
+    ax[2].set_xlabel('Score', fontdict=fontdict)
+    ax[2].grid(linestyle='--', alpha=0.5)
+    ax[2].legend(loc='upper left', fontsize='x-large')
+
+    twinx_ax2 = ax[2].twinx()
+    twinx_ax2.plot(x_linsrange[1:],
+                   regroup_badrate['bad_rate'].to_list(),
+                   linestyle='--',
+                   linewidth=linewidth,
+                   marker='x',
+                   markersize=markersize,
+                   markeredgewidth=5,
+                   c='r',
+                   label='Badrate')
+
+    twinx_ax2.set_ylabel('Badrate', fontdict=fontdict)
+    twinx_ax2.legend(loc='upper right', fontsize='x-large')
+
+    # 铺满整个画布
+    fig.tight_layout()
\ No newline at end of file
